/**
 * Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the AvrcX MTOS
 * 
 * Author: Winter Hu  <winter.hu@gmail.com>
 * Create: Dec 27, 2006
 */

#include "hardware.h"
#include "display.h"

#if defined DISPLAY

DISPBUFFER(disp, DISPLAY_COLS, DISPLAY_ROWS);

/**
 * Initialize display module
 */
INTERFACE void init_display(void){

  _drv_init_display();

  disp_clrbuf();
}

/**
 * Get current cursor
 *
 * @return Point*, The pointer to the cursor struct
 */
INTERFACE Point* disp_cursor(void){
  return &(disp.cursor);
}

/**
 * Move cursor to new site
 *
 * @param unsigned char x, The point x
 * @param unsigned char y, The point y
 * @return Point*, The pointer to the cursor struct
 */
INTERFACE Point* disp_moveto(unsigned char x, unsigned char y){
  disp.cursor.x = (x < disp.cols) ? x : disp.cols-1;
  disp.cursor.y = (y < disp.rows) ? y : disp.rows-1;
  return &(disp.cursor);  
}

/**
 * Write a character to display buffer of current cursor
 *
 * @param unsigned char b, The character
 */
INTERFACE void disp_putchar(unsigned char b){
  waitObject(&(disp.mutex));

  #if defined DISPLAY_CRLF
  if (b == '\r') disp.cursor.x = 0;
  if (b == '\n') 
    disp.cursor.y = (disp.cursor.y < disp.rows-1) ? disp.cursor.y+1 : 0;
  #endif

  *(disp.data + disp.cursor.x + disp.cursor.y*disp.cols) = b;

  disp.cursor.x++;
  if (disp.cursor.x >= disp.cols){
    disp.cursor.x = 0;
    disp.cursor.y = (disp.cursor.y < disp.rows-1) ? disp.cursor.y+1 : 0;
  }

  releaseMutex(&(disp.mutex));
}

/**
 * Clear all display page
 */
INTERFACE void disp_clrbuf(void){
  waitObject(&(disp.mutex));
  for(unsigned int i = 0; i < DISPLAY_BUFSIZE; i++){
    *(disp.data+i) = DISPLAY_BLANK;
  }
  releaseMutex(&(disp.mutex));
}

/**
 * Display current page
 * User should invokes this method in timer to reflash screen
 */
INTERFACE void display(void){
  waitObject(&(disp.mutex));

  for(unsigned char i = 0; i< disp.rows; i++){
    for(unsigned char j = 0; j < disp.cols; j++){
      _drv_display(i, j, *(disp.data+j+i*disp.cols));
    }
  }

  releaseMutex(&(disp.mutex));

}

#endif /* DISPLAY */
