/**
 * Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the AvrcX MTOS
 * 
 * Author: Winter Hu  <winter.hu@gmail.com>
 * Create: Dec 27, 2006
 */


#ifndef __DISPLAY_H__
#define __DISPLAY_H__

#include "common.h"
#include "sync.h"

#define DISPLAY_BUFSIZE DISPLAY_ROWS*DISPLAY_COLS

#ifndef __ASSEMBLER__
/* These only work in C program */

typedef struct Point {
  unsigned char x;
  unsigned char y;
} Point;

typedef struct CharacterDisplay {
  unsigned char cols;
  unsigned char rows;
  Point cursor;
  unsigned char *data;
  Mutex mutex;
} CharacterDisplay;

#define DISPBUFFER(name, cols, rows) 	  \
  unsigned char name##_data[cols*rows];   \
  CharacterDisplay name = {               \
    cols, rows,   			  \
    {0, 0},                               \
    &name##_data[0],                      \
    {TYPE_MUTEX, 1, {NULL}}		  \
  }

/**
 * Initialize display module
 */
INTERFACE void init_display(void);

/**
 * Get current cursor
 *
 * @return Point*, The pointer to the cursor struct
 */
INTERFACE Point* disp_cursor(void);

/**
 * Move cursor to new site
 *
 * @param unsigned char x, The point x
 * @param unsigned char y, The point y
 * @return Point*, The pointer to the cursor struct
 */
INTERFACE Point* disp_moveto(unsigned char x, unsigned char y);

/**
 * Write a character to display buffer of current cursor
 *
 * @param unsigned char b, The character
 */
INTERFACE void disp_putchar(unsigned char b);

/**
 * Clear all display page
 */
INTERFACE void disp_clrbuf(void);

/**
 * Display current page
 * User should invokes this method in timer to reflash screen, 
 * For example, 10ms
 */
INTERFACE void display(void);

/* Need device driver to implement */
extern void _drv_init_display(void);

extern void _drv_display(unsigned char row, unsigned char col, unsigned char b);

#endif /* !__ASSEMBLER__ */


#endif /* __DISPLAY_H__ */



