/**
 * Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the AvrcX MTOS
 * 
 * Author: Winter Hu  <winter.hu@gmail.com>
 * Create: Dec 26, 2006
 */


#include "hardware.h"
#include "keyboard.h"

#if defined KEYBOARD

#define KBD_STATUS_NOKEY 0
#define KBD_STATUS_DOWN  1
#define KBD_STATUS_DOWN1 2
#define KBD_STATUS_UP    3

unsigned char kbd_status = KBD_STATUS_NOKEY;
unsigned char kbd_keytmp = 0xFF;

INTERFACE void init_keyboard(void){
  #if KEYBOARD_UKBD == 1
  _drv_init_ukbd();
  #endif

  #if KEYBOARD_RKBD == 1
  _drv_init_rkbd();
  #endif
}

/**
 * Scan keyboard, this method should be invoked every 30ms
 *
 * @return unsigned char, The key value, if 0xFF, no key down
 */
INTERFACE unsigned char kbdscan(void){
  unsigned char key;

  #if KEYBOARD_UKBD == 1
  key = _drv_ukbdscan();
  switch(kbd_status){
  case KBD_STATUS_NOKEY:
    if (key != kbd_keytmp){
      kbd_keytmp = key;
      kbd_status = KBD_STATUS_DOWN;
      key = 0xFF;
    }
    break;
  case KBD_STATUS_DOWN:
    if (key != kbd_keytmp){
      kbd_keytmp = 0xFF;
      kbd_status = KBD_STATUS_NOKEY;
      key = 0xFF;
    }else{
      kbd_status = KBD_STATUS_DOWN1;
    }
    break;
  case KBD_STATUS_DOWN1:
    if (key != kbd_keytmp){
      kbd_status = KBD_STATUS_UP;
      key = kbd_keytmp;
    }
    break;
  case KBD_STATUS_UP:
    if (key != kbd_keytmp){
      kbd_status = KBD_STATUS_NOKEY;
      kbd_keytmp = 0xFF;
      key = 0xFF;
    }else{
      kbd_status = KBD_STATUS_DOWN1;
    }
    break;
  }
  #endif

  #if KEYBOARD_RKBD == 1
  key = (key == 0xFF) ? _drv_rkbdscan() : key;
  #endif

  return key;

}

#endif /* KEYBOARD */
