/**
 * Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * This file is part of the AvrcX MTOS
 * 
 * Author: Winter Hu  <winter.hu@gmail.com>
 * Create: Dec 25, 2006
 */


#ifndef __UART_H__
#define __UART_H__

#include "common.h"


#define UART_TXBUF_SIZE 16
#define UART_RXBUF_SIZE 16


#ifndef __ASSEMBLER__
/* These only work in C program */

/**
 * Initialize UART for Rx and Tx
 * You should invoke this method to initializ UART
 * Currently, this method just supported 8bit data, 1bit stop and none check
 *
 * @param unsigned int, UBRR = F_CPU/16/baudrate - 1
 */
INTERFACE void init_uart(unsigned int);

/**
 * Put a char to UART Tx queue
 * The byte maybe writed to UART directly, or be put into Uart's Tx queue
 * This method is blocked until the data had been put into queue
 *
 * @param char, the char need put to UART
 * @return 0 is successful, other values are failed
 */
INTERFACE int uart_putchar(char b);

/**
 * Get a char from UART Rx queue
 *
 * @return the char, if equas to -1, there are some errors occur.
 */
INTERFACE int uart_getchar(void);

#endif /* ! __ASSEMBLER__ */

#endif /* __UART_H__ */
