/**
 * Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * 
 * Author: Winter Hu  <winter.hu@gmail.com>
 * Create: Dec 26, 2006
 */

#include "hardware.h"
#include "../avrcx/avrcx.h"
#include <util/delay.h>
#include <avr/pgmspace.h>

#define SEG8_COMMON_PIN 0 // 0: Cathodic; 1: Anodic
#if SEG8_COMMON_PIN == 0
/* Encode list of  share cathodic of 8 segs led  */
#define SEG8_ENCODE_LIST(name)                      \
  const unsigned char FLASH name[] = {              \
    0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07, \
    0x7F, 0x6F, 0x77, 0x7C, 0x39, 0x5E, 0x79, 0x71, \
    0x00,                                           \
    0x40,				            \
    0x73                                            \
  }
#elif SEG8_COMMON_PIN == 1
/* Encode list of share anodic of 8 segs led */
#define SEG8_ENCODE_LIST(name)                      \
  const unsigned char FLASH name[] = {              \
    0xC0, 0xF9, 0xA4, 0xB0, 0x99, 0x92, 0x82, 0xF8, \
    0x80, 0x90, 0x88, 0x83, 0xC6, 0xA1, 0x86, 0x8E, \
    0xFF,                                           \
    0xBF,                                           \
    0x8C                                            \
  }
#endif

void _drv_init_display(void);

void _drv_display(unsigned char row, unsigned char col, unsigned char b);


// Allocate encode list space
SEG8_ENCODE_LIST(seg8_encoder);


void _drv_init_display(void){
  DDRC = 0xFF;
  PORTC= 0x00;

  DDRD  |= 0xF0;
  PORTD |= 0xF0;
}

void _drv_display(unsigned char row, unsigned char col, unsigned char b){
  PORTD |= 0xF0;
  clrBit(PORTD, 7-col);
  PORTC = pgm_read_byte(&seg8_encoder[b]);
  //PORTC = 0x77;
  _delay_ms(1);
  PORTD |= 0xF0;
}

