/**
 * Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf.
 * All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 *
 * 
 * Author: Winter Hu  <winter.hu@gmail.com>
 * Create: Dec 26, 2006
 */


#include "hardware.h"

#include "../avrcx/avrcx.h"
#include "../avrcx/display.h"
#include "../avrcx/keyboard.h"
#include "../avrcx/uart.h"
#include "../avrcx/util.h"

#include <avr/wdt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

const prog_uchar copyright[] = "Copyright (c) 2006-2008 iWESUN (ShenZhen) Inf. \r\n";
const prog_uchar product[] = "KBD3X4 v1.0 \r\n";
const prog_uchar contact[] = "Tel: 0755-29785775 \r\n";


TASKDEF(taskKbdscan, 16, 10, 10);
TASKDEF(taskDisplay, 16, 11, 10);
TASKDEF(taskCmdsend, 16, 11, 10);
TASKDEF(taskUartrcv, 16,  8, 10);

BYTEFIFO(kmsg, 8);

int main(void){
  wdt_disable();

  init_uart(UART_UBRR);

  init_display();

  init_keyboard();

  initScheduler();

  runTask(getTCB(taskKbdscan));  
  runTask(getTCB(taskDisplay));
  runTask(getTCB(taskCmdsend));
  runTask(getTCB(taskUartrcv));

  startScheduler();

}

/* Every 10ms reflash display buffer */
void taskDisplay(void){
  while(1){
    display();
    sleep(4);
  }
}

/* Every 30ms */
unsigned char _gKey = 0xFF;
void taskKbdscan(void){
  //  wdt_enable(WDTO_1S);

  while(1){
    unsigned char k = kbdscan();
    if (k != _gKey){
      if (_gKey == 0xFF){
	// Key down
	if (k == 0x2A){
	  disp_putchar(17);
	}else if (k == 0x23){
	  disp_putchar(15);
	}else{
	  unsigned char tmpk = (k >= 0x30) ? k-0x30 : k;
	  disp_putchar(tmpk);
	}
      }else{
	// Key up
	putByte(&kmsg, _gKey);
      }
      _gKey = k;
    }

    //wdt_reset();
    sleep(27);
  }
}


void taskCmdsend(void){

  print_pgm2uart(copyright);
  print_pgm2uart(product);
  print_pgm2uart(contact);

  while(1){
    
    unsigned char key = getByte(&kmsg);
    //

  }
}


void taskUartrcv(void){
  while(1){
    unsigned char b = uart_getchar();
    uart_putchar(b);
  }
}




